/*
 *    L I N K E D   L I S T   T O O L K I T
 *
 *    Copyright (C) 1990 by Garyl Lee Hester
 *
 *    llins.c
 *
 *    Insert and Move Routines
 */
#include <stdio.h>            /* for NULL definition */
#include "ll.h"

/*
 *    move pSrcAtom before/after pTrgAtom
 */
void ll_mvatom( PATOM    pSrcAtom,
                unsigned usDirection,
                PATOM    pTrgAtom,
                PLLENDS  pEnds
              )
{
  /*
   *     make sure that all pointers passed are not NULL, and that 
   *     source and target are not the same atom...
   */
   if( pSrcAtom && pTrgAtom && pEnds && pSrcAtom != pTrgAtom )
   {
      ll_unbatom( pSrcAtom, pEnds );
      ll_insatom( pSrcAtom, usDirection, pTrgAtom, pEnds );
   }
}

/*
 *    ll_insatom - insert pSrcAtom before/after pTrgAtom in list
 *                 defined by pEnds
 */
void ll_insatom( PATOM    pSrcAtom,
                 unsigned usDirection,
                 PATOM    pTrgAtom,
                 PLLENDS  pEnds
               )
{
   PLLBIND  pSrcBind;
   PLLBIND  pTrgBind;

  /*
   *     make sure that all pointers passed are not NULL, and that 
   *     source and target are not the same atom...
   */
   if( pSrcAtom && pTrgAtom && pEnds && pSrcAtom != pTrgAtom )
   {
      pSrcBind = MAKEPBIND( pSrcAtom );
      pTrgBind = MAKEPBIND( pTrgAtom );

      if( usDirection == _LL_AFTER )
      {
         if( pTrgBind->pNext != NULL )
            pTrgBind->pNext->pPrev = pSrcBind;
         pSrcBind->pNext = pTrgBind->pNext;
         pTrgBind->pNext = pSrcBind;
         pSrcBind->pPrev = pTrgBind;
         if( pEnds->pTail == (PGEN)pTrgBind )
            pEnds->pTail = (PGEN)pSrcBind;
      }
      else if( usDirection == _LL_BEFORE ) 
      {
         if( pTrgBind->pPrev != NULL )
            pTrgBind->pPrev->pNext = pSrcBind;
         pSrcBind->pPrev = pTrgBind->pPrev;
         pTrgBind->pPrev = pSrcBind;
         pSrcBind->pNext = pTrgBind;
         if( pEnds->pHead == (PGEN)pTrgBind )
            pEnds->pHead = (PGEN)pSrcBind;
      }
   }
}

