/*
 *    L I N K E D   L I S T   T O O L K I T
 *
 *    Copyright (C) 1990 by Garyl Lee Hester
 *
 *    llprim.c
 *
 *    Routines for Linked List Primitives
 */
#include <stdio.h>
#ifdef __TURBOC__
#include <mem.h>
#include <alloc.h>
#else
#include <memory.h>
#include <malloc.h>
#endif
#include "ll.h"

/*
 *    ll_open - initialize a new linked list control
 */
PLLCB ll_open( PLLCB pLL, unsigned usAtomSize, unsigned usAtomCnt )
{
   unsigned usRet = 0;

   if( pLL )
   {
     /*
      *     if the list is already active, destroy it
      */
      if( pLL->Block.pHead )
         ll_close( pLL );

      memset( (PGEN)pLL, 0x00, sizeof( LLCB ) );
      pLL->usAtomSize = usAtomSize;
      pLL->usAtomCnt  = usAtomCnt; 
   }
   return( pLL );
}

/*
 *    ll_close - closes a linked list
 */
void ll_close( PLLCB pLL )
{
   if( pLL )
   {
      while( pLL->Block.pHead )
         ll_rmblock( pLL, (PLLBC)( pLL->Block.pHead ) );
      memset( (PGEN)pLL, 0x00, sizeof( LLCB ) );
   }
}

/*
 *    ll_mklist - make a new sub list
 */
PLLST ll_mklist( PLLCB pLL, PLLST pList )
{
   if( pList && pLL )
   {
      pList->pLL        = pLL;
      pList->Ends.pHead = NULL;
      pList->Ends.pTail = NULL;
   }
   return( pList );
}

/*
 *    ll_rmlist - destroys a sub list
 */
void ll_rmlist( PLLST pList )
{
   if( pList )
   {
      while( pList->Ends.pHead )
         ll_free( pList, MAKEPATOM( pList->Ends.pHead ) );
   }
}

/*
 *    ll_alloc - return ptr to new atom, else NULL
 */
PATOM ll_alloc( PLLST pList )
{
   PLLBIND  pRet = NULL;
   PLLBC    pBlock;
   PLLCB    pLL;

   if( pList )
   {
      pLL = pList->pLL;
     /*
      *     if pFreeHead is NULL, then a new block needs to be created.
      */
      if( pLL->Free.pHead == NULL )
         ll_mkblock( pLL );

     /*
      *     if pFreeHead is STILL NULL, then there is no more memory
      */
      if( pLL->Free.pHead )
      {
         pRet = (PLLBIND)( pLL->Free.pHead );
         if( ( pBlock = ll_inblock( pLL, pRet ) ) )
            pBlock->usUsageCnt++;

         ll_unbatom( MAKEPATOM( pRet ), &pLL->Free );
         ll_appatom( MAKEPATOM( pRet ), &pList->Ends );
         pRet = MAKEPATOM( pRet );
      }
   }
   return( (PATOM) pRet );
}

/*
 *    ll_free - remove pAtom from pList and add to global free chain
 */
void ll_free( PLLST pList, PATOM pAtom )
{
   PLLBIND  pBind;
   PLLBC    pBlock;
   PLLCB    pLL;

   if( pList && pAtom )
   {
      pLL   = pList->pLL;
      pBind = MAKEPBIND( pAtom );
      ll_unbind( pList, pAtom );
      ll_appatom( pAtom, &pLL->Free );

      if( ( pBlock = ll_inblock( pLL, pAtom ) ) )
         if( !( --pBlock->usUsageCnt ) )
            ll_rmblock( pLL, pBlock );
   }
}


